"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.default = void 0;
var _reporter = _interopRequireDefault(require("gatsby-cli/lib/reporter"));
var _sourceNodesApiRunner = require("./source-nodes-api-runner");
var _redux = require("../redux");
var _datastore = require("../datastore");
var _actions = require("../redux/actions");
const {
  deleteNode
} = _actions.actions;

// console.log(`ya`, nodeOwnerSet.keys())
// /**
//  * Finds the name of all plugins which implement Gatsby APIs that
//  * may create nodes, but which have not actually created any nodes.
//  * owners are added to nodeOwnerSet from within the createNode action
//  */
// function discoverPluginsWithoutNodes(storeState: IGatsbyState): Array<string> {
//   return storeState.flattenedPlugins
//     .filter(
//       plugin =>
//         // "Can generate nodes"
//         plugin.nodeAPIs.includes(`sourceNodes`) &&
//         // "Has not generated nodes"
//         !nodeOwnerSet.has(plugin.name)
//     )
//     .map(plugin => plugin.name)
// }

/**
 * Warn about plugins that should have created nodes but didn't.
 */
// function warnForPluginsWithoutNodes(state: IGatsbyState): void {
//   console.log(`ya2`, nodeOwnerSet.keys())
//   const pluginsWithNoNodes = discoverPluginsWithoutNodes(state)

//   pluginsWithNoNodes.map(name =>
//     report.warn(
//       `The ${name} plugin has generated no Gatsby nodes. Do you need it? This could also suggest the plugin is misconfigured.`
//     )
//   )
// }

/**
 * Return the set of nodes for which its root node has not been touched
 */
function getStaleNodes(state, nodes) {
  return nodes.filter(node => {
    let rootNode = node;
    let next = undefined;
    let whileCount = 0;
    do {
      next = rootNode.parent ? (0, _datastore.getNode)(rootNode.parent) : undefined;
      if (next) {
        rootNode = next;
      }
    } while (next && ++whileCount < 101);
    if (whileCount > 100) {
      console.log(`It looks like you have a node that's set its parent as itself`, rootNode);
    }
    if (state.touchNodeOptOutTypes.has(rootNode.internal.type)) {
      return false;
    }
    return !state.nodesTouched.has(rootNode.id);
  });
}

/**
 * Find all stale nodes and delete them
 */
async function deleteStaleNodes(state, initialTypeNames) {
  let deleteCount = 0;
  const cleanupStaleNodesActivity = _reporter.default.createProgress(`clean up stale nodes`);
  cleanupStaleNodesActivity.start();
  for (const typeName of initialTypeNames) {
    if (state.touchNodeOptOutTypes.has(typeName)) {
      continue;
    }
    _reporter.default.info(`checking for ${typeName} nodes to delete`);
    const nodes = (0, _datastore.getDataStore)().iterateNodesByType(typeName);
    const staleNodes = getStaleNodes(state, nodes);
    const initialCount = deleteCount;
    for (const node of staleNodes) {
      _redux.store.dispatch(deleteNode(node));
      cleanupStaleNodesActivity.tick();
      if (++deleteCount % 1000) {
        // dont block event loop
        await new Promise(res => {
          setImmediate(() => {
            res(null);
          });
        });
      }
    }
    const amountDeleted = deleteCount - initialCount;
    _reporter.default.verbose(`Deleted ${amountDeleted} stale ${typeName} nodes`);
  }
  cleanupStaleNodesActivity.end();
}
let isInitialSourcing = true;
let sourcingCount = 0;
var _default = async ({
  webhookBody,
  pluginName,
  parentSpan,
  deferNodeMutation = false
}) => {
  await (0, _datastore.getDataStore)().ready();
  const potentiallyStaleNodeTypes = [];
  for (const typeName of (0, _datastore.getDataStore)().getTypes()) {
    potentiallyStaleNodeTypes.push(typeName);
  }
  const traceId = isInitialSourcing ? `initial-sourceNodes` : `sourceNodes #${sourcingCount}`;
  await (0, _sourceNodesApiRunner.sourceNodesApiRunner)({
    traceId,
    deferNodeMutation,
    parentSpan,
    webhookBody,
    pluginName
  });
  await (0, _datastore.getDataStore)().ready();

  // We only warn for plugins w/o nodes and delete stale nodes on the first sourcing.
  if (isInitialSourcing) {
    const state = _redux.store.getState();

    // Pass type names †hat existed before sourcing because no need to check for types that didn't exist before. none of those are stale.
    await deleteStaleNodes(state, potentiallyStaleNodeTypes);
    // warnForPluginsWithoutNodes(state)

    isInitialSourcing = false;
  }
  _redux.store.dispatch(_actions.actions.apiFinished({
    apiName: `sourceNodes`
  }));
  sourcingCount += 1;
};
exports.default = _default;
//# sourceMappingURL=source-nodes.js.map